# -*- coding: utf-8 -*-
# Copyright (C) 2006, 2007, 2008 Stewart Adam
# This file is part of fwfstab.

# fwfstab is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# fwfstab is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with fwfstab; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Parts taken or adapted from `hwbrowser' - Copyright 2001 Red Hat, Inc.

"""
fstab: A python library to interface with the fstab.
module variables: 
"""
import kudzu
import parted
import subprocess
import os, os.path

from fwfstab.i18n import _

__author__ = "Stewart Adam <s.adam@diffingo.com>"
__status__ = "beta"
__version__ = "0.03"
__license__ = "GNU GPLv2+"

def execute(command, env=None, shell=False, timeout=-1):
  """ Execute a command, wait for it to finish """
  sub = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell, env=env)
  return sub.wait(), sub.stdout, sub.stderr

def executeSub(command, env=None, shell=False):
  """ Execute a command async """
  sub = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell, env=env)
  return sub

def CheckPerms(path):
  """ Checks for read and write permissions.
      Takes the path to check as an argument, returns
      True if permissions check out, False if not.
  """
  read = CheckPermsRead(path)
  write = CheckPermsWrite(path)
  if read == True and write == True:
      return True
  return False

def CheckPermsRead(path):
  """ Checks for read permissions.
      Takes the path to check as an argument, returns
      True if permissions check out, False if not.
  """
  if not os.path.exists(path):
    path = os.path.dirname(path)
  return os.access(path, os.R_OK)

def CheckPermsWrite(path):
  """ Checks for write permissions.
      Takes the path to check as an argument, returns
      True if permissions check out, False if not.
  """
  if not os.path.exists(path):
    path = os.path.dirname(path)
  return os.access(path, os.W_OK)

# -- Hard drive logic --

def getHardDrive(node):
  """ Return a list of a hard drive with node `node' """
  #print _('Autodetecting drives...')
  for harddisk in kudzu.probe(kudzu.CLASS_HD, kudzu.BUS_UNSPEC, kudzu.PROBE_SAFE):
    desc = harddisk.desc
    devnode = harddisk.device
    if devnode == None or devnode != node:
      continue
    else:
      disk = HardDisk(desc, devnode)
      return disk
    return None

def getHardDrives():
  """ Return a list of all hard drive names """
  disks = []
  #print _('Autodetecting drives...')
  for harddisk in kudzu.probe(kudzu.CLASS_HD, kudzu.BUS_UNSPEC, kudzu.PROBE_SAFE):
    desc = harddisk.desc
    devnode = harddisk.device
    if devnode == None:
      continue
    disk = HardDisk(desc, devnode)
    disks.append(disk)
  return disks

def getPartitionType(partition):
  """ Return the FS on a partition """
  if partition.type & parted.PARTITION_FREESPACE:
      fs_type = _("Free space")
  elif partition.type & parted.PARTITION_EXTENDED:
      fs_type = _("Extended")
  elif partition.get_flag(parted.PARTITION_RAID) == 1:
      fs_type = _("Software RAID component")
  elif partition.get_flag(parted.PARTITION_LVM) == 1:
      fs_type = _("LVM Physical Volume")
  elif partition.fs_type:
      fs_type = partition.fs_type.name
  else:
      fs_type = _("No filesystem")
  return fs_type

def getPartitionNode(partition):
  if (partition.geom.dev.type == parted.DEVICE_DAC960 or partition.geom.dev.type == parted.DEVICE_CPQARRAY):
    return "/dev/%sp%d" % (partition.geom.dev.path[5:], partition.num)
  return "/dev/%s%d" % (partition.geom.dev.path[5:], partition.num)

class HardDisk:
  """ Represents a physical harddisk """
  def __init__(self, desc, devnode):
    """ Initialize the parted instances """
    self.devnode = devnode
    self.desc = desc
    self.partEd = parted.PedDevice.get('/dev/%s' % devnode)
    self.partDisk = parted.PedDisk.new(self.partEd)

  def getPartitions(self):
    """ Return a list of all partitions on a given devnode """
    partitions = []
    part = self.partDisk.next_partition()
    while part:
      if part.type_name == "metadata" or part.type_name == "free":
        part = self.partDisk.next_partition(part)
        continue
      #print part.num, part.type, part.type_name, part.fs_type
      partitions.append(part)
      # finally...
      part = self.partDisk.next_partition(part)
    return partitions

  def getPartitionLength(self, partition):
    """ Return the length of a partition in MB """
    return round((partition.geom.length * self.partEd.sector_size / 1024.0 / 1024.0), 1)

class Dialog:
    """ Wrapper for error dialog """
    def __init__(self, widget, ffilter=None):
        """ Initialize.
            widget: The widget to use
        """
        self.dialog = widget
    def run(self):
        """ Run the dialog """
        self.dialog.show()
        try:
            return self.dialog.run()
        except:
            self.destroy()
            raise
    def destroy(self):
        """ Destroy the dialog and the filter """
        self.dialog.hide()

class PathBrowser:
    """ Wrapper for path dialog """
    def __init__(self, widget, ffilter=None):
        """ Initialize.
            widget: The widget to use
            ffilter: List for filter creation.
                    > 1st value is filter pattern
                    > 2nd value is the description
        """
        self.dialog = widget
        if ffilter:
            self.ffilter = gtk.FileFilter()
            self.ffilter.add_pattern(ffilter[0])
            self.ffilter.set_name(ffilter[1])
            self.dialog.add_filter(self.ffilter)
        else:
            self.ffilter = None

    def run(self):
        """ Run the dialog """
        self.dialog.show()
        try:
            return self.dialog.run()
        except:
            self.destroy()
            raise
    def destroy(self):
        """ Destroy the dialog and the filter """
        self.dialog.hide()
        if self.ffilter:
            self.dialog.remove_filter(self.ffilter)
    def set_current_folder(self, path):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_current_folder(path)
    #gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER
    #gtk.FILE_CHOOSER_ACTION_OPEN
    def set_select_multiple(self, bool):
        """ Wrapper: See GTK+ help. """
        if bool == True:
            return self.dialog.set_select_multiple(True)
        elif bool == False:
            return self.dialog.set_select_multiple(False)
    def set_action(self, action):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_action(action)
    def set_title(self, title):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_title(title)
    def get_filename(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def get_filenames(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def set_filename(self, filename):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_filename(filename)
    def add_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.add_filter(ffilter)
    def remove_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.remove_filter(ffilter)


